@@ -8,6 +8,7 @@ import sys
 import builtins
 import logging
 import operator
+import re
 
 from .funcchecker import FuncLister, list_documented_parameters, max_documented_returns, CallLister, ModuleCallLister
 from .classchecker import ClassPropertiesLister
@@ -77,6 +78,8 @@ def get_assigned_ids(node):
                 yield target.asname
             else:
                 yield target.name
+        else:
+            yield target.name.split('.')[0]
     if hasattr(node, 'elts'):
         for el in node.elts:
             yield from get_assigned_ids(el)
@@ -94,7 +97,10 @@ def get_deleted_ids(node):
 
 
 def is_container(node):
-    """Check if node contains code."""
+    """Check if node contains code.
+
+    :param node: AST node
+    """
     return isinstance(node, ast.FunctionDef) or isinstance(node, ast.AsyncFunctionDef) or isinstance(node, ast.ClassDef) or isinstance(node, ast.Lambda) or isinstance(node, ast.Expr) or isinstance(node, ast.Call)
 
 
@@ -143,7 +149,10 @@ def get_all_ids(node):
 
 
 def block_terminates(statements):
-    """Check if code block stops the execution."""
+    """Check if code block stops the execution.
+
+    :param statements: AST nodes
+    """
     for statement in statements:
         if isinstance(statement, ast.Return):
             return True
@@ -164,39 +173,108 @@ class FuncDocVerifier(ast.NodeVisitor):
         self.class_methods = []
         self.checked_docstrings = 0
 
+    KNOWN_DIRECTIVES = ':mod: :func: :data: :const: :class: :meth: :attr: :exc: :obj:'.split()
+
     def visit_ClassDef(self, node):
         for child in ast.iter_child_nodes(node):
             # look for methods:
             if isinstance(child, ast.FunctionDef):
                 self.class_methods.append(child)
+        docstring = ast.get_docstring(node, clean=True)
+        if docstring is None:
+            self.log.debug('%s:%d:no docstring for class "%s"', self.filename, node.lineno, node.name)
+            return
+        self.find_directives(docstring, node.name, node.body[0].value.lineno)
         self.generic_visit(node)
 
+    def find_directives(self, docstring, nodename, lineno, outstream=sys.stderr):
+        """find faulty python rst referencing directives.
+
+        * tests for rst links with no _ at the end, or no space between text and URI.
+        * tests for unknown `py` directives, and `py` directives that are placed in global domain.
+        * tests for `py` directives that lack quotes
+
+        Parameters
+        ----------
+        docstring: str
+            string to test
+        nodename: str
+            name of node where the docstring came from
+        lineno: int
+            Line number
+        outstream: File
+            Output file to write output information to
+        """
+        if '<' in docstring:
+            pattern = r'`.*[^ ]( ?)<[^>]*>`(_?)'
+            for i, line in enumerate(docstring.split('\n')):
+                if '<' not in line:
+                    continue
+                matches = re.finditer(pattern, line)
+                for match in matches:
+                    if match.group(1) == '':
+                        outstream.write('%s:%d: docstring WARNING: rst link needs space before <url>.\n' % (
+                            self.filename, lineno + i))
+                    if match.group(2) == '':
+                        outstream.write('%s:%d: docstring WARNING: rst link should end with _.\n' % (
+                            self.filename, lineno + i))
+
+        if ':' in docstring:
+            pattern = r'(:py)?(:?[a-z]{1,20}:?)(`?)'
+            for i, line in enumerate(docstring.split('\n')):
+                if ':' not in line:
+                    continue
+                matches = re.finditer(pattern, line)
+                for match in matches:
+                    # print('matchgroup: "%s" "%s" "%s"' % (match.group(0), match.group(1), match.group(2)))
+                    if match.group(1) is None and match.group(2) in self.KNOWN_DIRECTIVES:
+                        outstream.write('%s:%d: docstring WARNING: "%s" should be ":py%s".\n' % (
+                            self.filename, lineno + i, match.group(2), match.group(2)))
+                    elif match.group(1) == ':py':
+                        if match.group(2) not in self.KNOWN_DIRECTIVES:
+                            outstream.write('%s:%d: docstring WARNING: unknown directive "%s%s".\n' % (
+                                self.filename, lineno + i, match.group(1), match.group(2)))
+                        elif match.group(3) != '`':
+                            outstream.write('%s:%d: docstring WARNING: directive should continue with `quotes`.\n' % (
+                                self.filename, lineno + i))
+
     def visit_FunctionDef(self, node):
         arguments = node.args
+        # allow "Parameters" as first line as well by prepending a newline
         func_docstring = ast.get_docstring(node, clean=True)
         if func_docstring is None:
             self.log.debug('%s:%d:no docstring for function "%s"', self.filename, node.lineno, node.name)
             return
+        self.find_directives(func_docstring, node.name, node.body[0].value.lineno)
 
-        documented_parameters = list_documented_parameters(func_docstring)
+        documented_parameters = list_documented_parameters('\n' + func_docstring)
         function_arguments = [arg.arg for arg in arguments.args]
+        variable_length = arguments.vararg or arguments.kwarg
         if node in self.class_methods:
-            arguments = function_arguments[1:]
+            argument_names = function_arguments[1:]
         else:
-            arguments = function_arguments
+            argument_names = function_arguments
         del function_arguments
         if len(documented_parameters) == 0:
-            if len(arguments) > 0:
+            if len(argument_names) > 0 and not node.name.startswith('_'):
                 sys.stderr.write('%s:%d: WARNING: function "%s" does not have any parameter docs\n' % (
                     self.filename, node.lineno, node.name))
             return
         self.checked_docstrings += 1
         self.log.debug("documented parameters of %s: %s", node.name, ', '.join(documented_parameters))
-        for arg in arguments:
+        for arg in argument_names:
             if arg not in documented_parameters:
                 sys.stderr.write('%s:%d: ERROR: argument "%s" of "%s" missing in docstring\n' % (
                     self.filename, node.lineno, arg, node.name))
                 self.undocumented_parameters_found |= True
+            del arg
+        if not variable_length:
+            for arg in documented_parameters:
+                if not arg.startswith('*') and arg not in argument_names:
+                    sys.stderr.write('%s:%d: ERROR: "%s" in docstring is not an argument of "%s"\n' % (
+                        self.filename, node.lineno, arg, node.name))
+                    self.undocumented_parameters_found |= True
+                del arg
 
         all_returns = [
             (return_tuple_length, return_node)
@@ -257,13 +335,31 @@ class NameAssignVerifier():
         self.known_checked = 0
 
     def variable_unknown_found(self, lineno, name):
-        """Report a undefined variable `name` was found in line `lineno`."""
+        """Report a undefined variable `name` was found in line `lineno`.
+
+        Parameters
+        ----------
+        lineno: int
+            Line number
+        name: str
+            variable name
+        """
         sys.stderr.write('%s:%d: ERROR: Variable unknown: "%s"\n' % (self.filename, lineno, name))
         self.found_variable_unknown |= True
         raise Exception(name)
 
     def assert_unknown(self, name, known, lineno, override_with_builtins=False):
-        """Report error if `name` is already in set `known`."""
+        """Report error if `name` is already in set `known`.
+
+        Parameters
+        ----------
+        lineno: int
+            Line number
+        name: str
+            variable name
+        override_with_builtins: bool
+            whether a variable assignment overriding a builtin name should raise an error
+        """
         self.log.debug("assert_unknown: %s, %s", name, lineno)
         assert name is not None
         # the nested if allows deleting builtins and re-defining them afterwards
@@ -289,7 +385,17 @@ class NameAssignVerifier():
             self.unknown_checked += 1
 
     def check_new_identifiers(self, elements, node, known):
-        """Add newly defined variables and verify that the accessed ones are defined in known"""
+        """Add newly defined variables and verify that the accessed ones are defined in known.
+
+        Parameters
+        ----------
+        elements: list
+            list of ast nodes of variables
+        node: Node
+            current node
+        known: set
+            known variable names
+        """
         add = {}
         forget = set()
         self.log.debug('check_new_identifiers: known: %s', known.keys() - preknown)
@@ -305,10 +411,10 @@ class NameAssignVerifier():
                         name = getattr(name, 'asname')
                     elif hasattr(name, 'name'):
                         name = name.name
-                    self.log.debug('check_new_identifiers: name %s', name)
+                    self.log.debug('check_new_identifiers: name %s from module %s', name, getattr(node, 'module', ''))
                     self.assert_unknown(
                         name, known, el.lineno,
-                        override_with_builtins=getattr(node, 'module', '') == 'builtins')
+                        override_with_builtins=getattr(node, 'module', '') in ('builtins', 'six.moves'))
                     name = name.split('.')[0]
                     add_here[name] = el.lineno
                 del name, names
@@ -480,6 +586,10 @@ class NameAssignVerifier():
                         if name not in known_nodes or known_nodes[name][0] not in (True, None):
                             self.variable_unknown_found(node.lineno, name)
                         known_nodes2[name] = (True, node.lineno)
+            for withitem in getattr(node, 'items', []):
+                name = withitem.optional_vars
+                if name is not None:
+                    known_nodes2[name.id] = (True, node.lineno)
             if hasattr(node, 'body'):
                 body = node.body if isinstance(node.body, list) else [node.body]
                 members = self.walk_tree(body, known_nodes2, depth + 1)
@@ -491,7 +601,6 @@ class NameAssignVerifier():
                 self.log.debug('%s+nodes from body: %s', depthstr, nodes_to_add)
 
             for handler in getattr(node, 'handlers', []):
-                members = self.walk_tree(handler.body, known_nodes, depth + 1)
                 if not block_terminates(handler.body):
                     nbranches += 1
                     known_nodes2 = dict(**known_nodes)
@@ -501,6 +610,7 @@ class NameAssignVerifier():
                     for name, var in members.items():
                         if name not in known_nodes or known_nodes[name][0] != var[0]:
                             nodes_to_add[name] = (nodes_to_add.get(name, (0, None))[0] + 1, var)
+                members = self.walk_tree(handler.body, known_nodes2, depth + 1)
                 self.log.debug('%s+nodes from handlers: %s', depthstr, nodes_to_add)
 
             if getattr(node, 'orelse', None) is not None:
@@ -583,16 +693,16 @@ def main(filenames, module_load_policy='none', allow_variable_reuse=False):
         nameassigner.walk_tree(a.body, known)
         total_checked_known_var += nameassigner.known_checked
         total_checked_unknown_var += nameassigner.unknown_checked
+        print("%s: checking docstrings ..." % filename)
+        funcdocs = FuncDocVerifier(filename=filename)
+        funcdocs.visit(a)
+        total_checked_docstrings += funcdocs.checked_docstrings
         if nameassigner.found_variable_unknown:
             sys.exit(2)
         if nameassigner.found_variable_reused and not allow_variable_reuse:
             sys.exit(3)
         if nameassigner.found_builtin_overwritten:
             sys.exit(4)
-        print("%s: checking docstrings ..." % filename)
-        funcdocs = FuncDocVerifier(filename=filename)
-        funcdocs.visit(a)
-        total_checked_docstrings += funcdocs.checked_docstrings
         if funcdocs.undocumented_parameters_found:
             sys.exit(5)
         if funcdocs.undocumented_returns_found:
